#include <windows.h>
#include <stdio.h>

SC_HANDLE schService =0;
SC_HANDLE schSCManager = 0 ;

BOOL FileExists(char *lpszFilePath){

  WIN32_FIND_DATA FindFileData;
  HANDLE hFind;  
  
  hFind = FindFirstFile(lpszFilePath, &FindFileData);
  if (hFind == INVALID_HANDLE_VALUE) 
  {
    printf ("Invalid File Handle. GetLastError reports %d\n", GetLastError ());
    return false;
  } 
  else 
  {
    //printf ("The first file found is %s\n", FindFileData.cFileName);
    FindClose(hFind);
    return true;
  }
}

BOOL DoScmOpen(){
	
	schSCManager = OpenSCManager( 
		NULL,                    // local machine 
		NULL,                    // ServicesActive database 
		SC_MANAGER_ALL_ACCESS);  // full access rights 

	if (NULL == schSCManager) {
		printf("OpenSCManager failed (%d)\n", GetLastError());
		return false;
	}
	else{
		return true;
	}
}


VOID DeleteSampleService() 
{ 

	schService = OpenService( 
        schSCManager,       // SCManager database 
        "SysConsole",       // name of service 
        DELETE);            // only need DELETE access 
 
    if (schService == NULL) 
        printf("OpenService Failed");  
 
    if (! DeleteService(schService) ) 
		printf("Delete Service Failed");
    else 
        printf("DeleteService SUCCESS\n");
	
	CloseServiceHandle(schService); 

 
}


BOOL InstallService() 
{ 
    TCHAR szPath[MAX_PATH]; 
	SC_HANDLE  schService = 0 ;
	
	char* lpszDisplayName = 0 ;

	GetCurrentDirectory(MAX_PATH,szPath);
    strcat(szPath,"\\SysConsole.exe");
	
	if(!FileExists(szPath)){return 0;}

    schService = CreateService( 
        schSCManager,              // SCManager database 
        TEXT("SysConsole"),        // name of service 
        lpszDisplayName,           // service name to display 
        SERVICE_ALL_ACCESS,        // desired access 
        SERVICE_WIN32_OWN_PROCESS, // service type 
        SERVICE_DEMAND_START,      // start type 
        SERVICE_ERROR_NORMAL,      // error control type 
        szPath,                    // path to service's binary 
        NULL,                      // no load ordering group 
        NULL,                      // no tag identifier 
        NULL,                      // no dependencies 
        NULL,                      // LocalSystem account 
        NULL);                     // no password 
 
    if (schService == NULL) 
    {
        printf("CreateService failed (%d)\n", GetLastError()); 
        return FALSE;
    }
    else
    {
        CloseServiceHandle(schService); 
        return TRUE;
    }


}

DWORD StartSampleService() 
{ 
    SERVICE_STATUS ssStatus; 
    DWORD dwOldCheckPoint; 
    DWORD dwStartTickCount;
    DWORD dwWaitTime;
    DWORD dwStatus;
 
    schService = OpenService( 
        schSCManager,          // SCM database 
        "SysConsole",          // service name
        SERVICE_ALL_ACCESS); 
 
    if (schService == NULL) 
    { 
        //MyErrorExit("OpenService"); 
		printf("Start::Open Service Failed:");
    }
 
    if (!StartService(
            schService,  // handle to service 
            0,           // number of arguments 
            NULL) )      // no arguments 
    {
        //MyErrorExit("StartService"); 
		printf("Start::StartFailed");
    }
    else 
    {
        printf("Service start pending.\n"); 
    }
 
    // Check the status until the service is no longer start pending. 
 
    if (!QueryServiceStatus( 
            schService,   // handle to service 
            &ssStatus) )  // address of status information structure
    {
        //MyErrorExit("QueryServiceStatus"); 
    }
 
    // Save the tick count and initial checkpoint.

    dwStartTickCount = GetTickCount();
    dwOldCheckPoint = ssStatus.dwCheckPoint;

    while (ssStatus.dwCurrentState == SERVICE_START_PENDING) 
    { 
        // Do not wait longer than the wait hint. A good interval is 
        // one tenth the wait hint, but no less than 1 second and no 
        // more than 10 seconds. 
 
        dwWaitTime = ssStatus.dwWaitHint / 10;

        if( dwWaitTime < 1000 )
            dwWaitTime = 1000;
        else if ( dwWaitTime > 10000 )
            dwWaitTime = 10000;

        Sleep( dwWaitTime );

        // Check the status again. 
 
        if (!QueryServiceStatus( 
                schService,   // handle to service 
                &ssStatus) )  // address of structure
            break; 
 
        if ( ssStatus.dwCheckPoint > dwOldCheckPoint )
        {
            // The service is making progress.

            dwStartTickCount = GetTickCount();
            dwOldCheckPoint = ssStatus.dwCheckPoint;
        }
        else
        {
            if(GetTickCount()-dwStartTickCount > ssStatus.dwWaitHint)
            {
                // No progress made within the wait hint
                break;
            }
        }
    } 

    if (ssStatus.dwCurrentState == SERVICE_RUNNING) 
    {
        printf("StartService SUCCESS.\n"); 
        dwStatus = NO_ERROR;
    }
    else 
    { 
        printf("\nService not started. \n");
        printf("  Current State: %d\n", ssStatus.dwCurrentState); 
        printf("  Exit Code: %d\n", ssStatus.dwWin32ExitCode); 
        printf("  Service Specific Exit Code: %d\n", 
            ssStatus.dwServiceSpecificExitCode); 
        printf("  Check Point: %d\n", ssStatus.dwCheckPoint); 
        printf("  Wait Hint: %d\n", ssStatus.dwWaitHint); 
        dwStatus = GetLastError();
    } 
 
    CloseServiceHandle(schService); 
    return dwStatus;
}

void Usage(void){
	 printf("Usage SysConsoleInstall.exe /run|install|remove");
	 exit(0);
}


int main(int argc, char* argv[])
{
  
   if(argc == 1){ Usage(); }
   
   if(strcmp(argv[1],"-?") == 0 || strcmp(argv[1],"?") ==0 || strcmp(argv[1],"/?") == 0) Usage();

   if(!DoScmOpen()){
	   printf("ScmOpen Failed");
	   return 0;
   }

   if(stricmp(argv[1],"run") == 0){
		 StartSampleService();
   }
   else if(stricmp(argv[1],"install")==0){
	     printf("Installed:? %d", InstallService() ); 
   }
   else if(stricmp(argv[1],"remove")==0){ 	 
		 DeleteSampleService();
		
   }

   return 0;
   
}

